#
#   Makefile.PL for Gwine
#    - Primary version make by Glade-Perl
#    - Modify by S. Daubert

eval { require 5.8.4 } or die <<EOD;
######################################################
## Gwine uses version 5.8.0 of Perl.                ##
## Please upgrade your Perl before installing Gwine ##
######################################################
EOD

use ExtUtils::MakeMaker;
use strict;
use warnings;

#--- Parse Args ---
our $install_gconf_schema = 1;
our $deb_pkg = 0;

my @new_args = ();
foreach (@ARGV) {
    if ($_ eq 'PKG=deb') {
	$install_gconf_schema = 0;
	$deb_pkg = 1;
    } else { unshift @new_args, $_ }
}
@ARGV = @new_args;

#--- Configuration section ---

my @programs_to_install = qw( gwine );

my @need_perl_modules = (
    # Check for Locale::TextDomain
    {'name'     => 'libintl-perl',
    'test'      => 'Locale::TextDomain',
    'version'   => '1.11',
    'reason'    => "implements the perl bindings to libintl\n".
                    "The module is called libintl-perl on CPAN"},

    # Check for Gtk2
    {'name'     => 'Gtk2',
    'test'      => 'Gtk2',
    'version'   => '1.081',
    'reason'    => "implements the perl bindings to Gtk+ 2.x.\n".
                    "The module is called Gtk2-Perl on CPAN"},

    # Check for Gtk2::GladeXML
    {'name'     => 'Gtk2::GladeXML',
    'test'      => 'Gtk2::GladeXML',
    'version'   => '1.003',
    'reason'    => "implements the perl bindings to libglade 2.x.\n".
                    "The module is called Gtk2-GladeXML on CPAN"},

    # Check for Gnome2
    {'name'     => 'Gnome2',
    'test'      => 'Gnome2',
    'version'   => '1.021',
    'reason'    => "implements the perl bindings to Gnome 2.x.\n".
                   "It is a submodule of the Gtk2-Perl package and needs to be built separately.\n".
                   "Read the Gtk2-Perl INSTALL file for details of how to do this."},

    # Check for Gnome2::GConf
    {'name'     => 'Gnome2::GConf',
    'test'      => 'Gnome2::GConf',
    'version'   => '1.000',
    'reason'    => "implements the perl bindings to GConf 2.x.\n".
                   "It is a submodule of the Gtk2-Perl package and needs to be built separately.\n".
                   "Read the Gtk2-Perl INSTALL file for details of how to do this."},

    # Check for DateTime
    {'name'     => 'DateTime',
    'test'      => 'DateTime',
    'version'   => '0.22',
    'reason'    => "manipulates date and time according to locale"},
    );

my @need_ext_programs = 
    (
     {'name'      => 'gettext',
      'test'      => 'xgettext --version',
      'version'   => '0.14.5',
      'reason'    => "localizes Gwine"
     },
     {'name'      => 'scrollkeeper',
      'test'      => 'scrollkeeper-config --version',
      'version'   => '0.3.14',
      'reason'    => "installs GNOME documentation"
     },
    );

#--- End Configuration - You should not have to change anything below this line

# Allow us to suppress all program installation with the -n (library only)
# option.  This is for those that don't want to mess with the configuration
# section of this file.
use Getopt::Std;
use vars qw($opt_n);
unless (getopts("n")) {
    die "Usage: $0 [-n]\n";
}
@programs_to_install = () if $opt_n;

# Check for non-standard modules that are used by this library.
$| = 1; # autoflush on
my $missing_modules = 0;

foreach my $mod (@need_perl_modules) {
    print "Checking for $mod->{'name'}..";
    eval "use $mod->{'test'} $mod->{'version'}";
    if ($@) {
        $missing_modules++;
		print " failed\n";
        print   "-------------------------------------------------------".
                "\n$@\n",
                "$mod->{'name'} is needed, it $mod->{'reason'}\n",
                "We need at least version $mod->{'version'}\n".
                "-------------------------------------------------------\n";
        sleep(2);  # Don't hurry too much
    } else {
		print " ok\n";
    }
}

print "-------------------------------------------------------
The missing modules can be obtained from CPAN. Visit
<URL:http://www.perl.com/CPAN/> to find a CPAN site near you.
-------------------------------------------------------\n\n"
     if $missing_modules;

#--------------------------------------

# Check for externam programs.
$| = 1; # autoflush on

foreach my $prog (@need_ext_programs) {
    print "Checking for $prog->{'name'}..";
    my $v = `$prog->{'test'}`;
    $v =~ m/(\d+\.\d+(\.\d+)?)/;
    $v = $1;
    if (!defined $v or $v lt $prog->{'version'}) {
	print " failed\n";
        print   "-------------------------------------------------------".
                "\n$@\n",
                "$prog->{'name'} is needed, it $prog->{'reason'}\n",
                "We need at least version $prog->{'version'}\n".
                "-------------------------------------------------------\n";
        sleep(2);  # Don't hurry too much
    } else {
	print " ok\n";
    }
}

#--------------------------------------
# Last of all generate the Makefile
my %PM = (
	'Gwine/Config.pm' => '$(INST_LIBDIR)/Gwine/Config.pm',
	'Gwine/DialogCellar.pm' => '$(INST_LIBDIR)/Gwine/DialogCellar.pm',
	'Gwine/DialogPref.pm' => '$(INST_LIBDIR)/Gwine/DialogPref.pm',
	'Gwine/DialogProd.pm' => '$(INST_LIBDIR)/Gwine/DialogProd.pm',
	'Gwine/DialogWine.pm' => '$(INST_LIBDIR)/Gwine/DialogWine.pm',
	'Gwine/GAppel.pm' => '$(INST_LIBDIR)/Gwine/GAppel.pm',
	'Gwine/GWine.pm' => '$(INST_LIBDIR)/Gwine/GWine.pm',
	'Gwine/GProd.pm' => '$(INST_LIBDIR)/Gwine/GProd.pm',
	'Gwine/GList.pm' => '$(INST_LIBDIR)/Gwine/GList.pm',
	'Gwine/GLAppel.pm' => '$(INST_LIBDIR)/Gwine/GLAppel.pm',
	'Gwine/GLWine.pm' => '$(INST_LIBDIR)/Gwine/GLWine.pm',
	'Gwine/GLProd.pm' => '$(INST_LIBDIR)/Gwine/GLProd.pm',
	'Gwine/GZone.pm' => '$(INST_LIBDIR)/Gwine/GZone.pm',
	'Gwine/GLZone.pm' => '$(INST_LIBDIR)/Gwine/GLZone.pm',
	'Gwine/Gdb.pm' => '$(INST_LIBDIR)/Gwine/Gdb.pm',
	'Gwine/gwine.pm' => '$(INST_LIBDIR)/Gwine/gwine.pm',
	'Gwine/L10N.pm' => '$(INST_LIBDIR)/Gwine/L10N.pm',
	'Gwine/L10N/fr.pm' => '$(INST_LIBDIR)/Gwine/L10N/fr.pm',
	'Gwine/L10N/de.pm' => '$(INST_LIBDIR)/Gwine/L10N/de.pm',
	'Gwine/L10N/cs.pm' => '$(INST_LIBDIR)/Gwine/L10N/cs.pm',
	 );

# If no debian package is going to be build, then install theese files
if (!$deb_pkg) {
    $PM{'Changelog'} = '$(INST_LIBDIR)/Gwine/Changelog';
    $PM{'COPYING'} = '$(INST_LIBDIR)/Gwine/COPYING';
    $PM{'COPYING-DOCS'} = '$(INST_LIBDIR)/Gwine/COPYING-DOCS';
    $PM{'README'} = '$(INST_LIBDIR)/Gwine/REAME';
    $PM{'README-NLS.gz'} = '$(INST_LIBDIR)/Gwine/README-NLS.gz';
}


WriteMakefile(
    'DISTNAME'     => 'gwine',
    'NAME'         => 'gwine',
    'VERSION_FROM' => 'Gwine/gwine.pm',
    'EXE_FILES'    => [ @programs_to_install ],
    'PREFIX'       => '/usr/local',
    'INSTALLDIRS'  => 'perl',
    'clean'        => { FILES => '*.out po/*.mo *~ Gwine/*~ Gwine/L10N/*~ po/*~' },
    'dist'         => { COMPRESS => 'gzip', SUFFIX => 'gz' },
    'PM'           => \%PM,
);

package MY;

use IO::Dir;

# Function to install a directory

sub install_dir {
    my $from = shift;
    my $to   = shift;

    my @res = ();
    my %dir;
    tie %dir, 'IO::Dir', $from;
    push @res, "\tinstall -d $to\n";
    foreach my $file (keys %dir) {
	next if $file eq '.';
	next if $file eq '..';
	next if $file eq 'CVS';
	if ($dir{$file}->mode & 0x4000 ) {
	     push @res, install_dir("$from/$file", "$to/$file")
	} else {
	     push @res, "\tinstall -m 644 $from/$file $to\n";
	}
    }
    return @res;
}


#######################################
## Functions from ExtUtils::MakeMaker

sub top_targets {
    my $self = shift;
    local($_) = $self->SUPER::top_targets(@_);
    s/all ::/all :: update_mo update_omf/m;
    $_;
}

# Pass version number to pod2man
sub manifypods
{
    my $self = shift;
    my $ver = $self->{'VERSION'} || "";
    local($_) = $self->SUPER::manifypods(@_);
    s/pod2man\s*$/pod2man --release gwine-$ver/m;
    $_;
}

sub install {
    my $self = shift;
    local($_) = $self->SUPER::install(@_);
    s/install ::/install:: post_install/m;
    s/uninstall ::/uninstall:: post_uninstall/m;
    $_;
}

sub postamble {
    my $self = shift;

    ## INSTALL_DIR = dir where data go
    my $INSTALL_DIRS = '';
    if (exists $ENV{XDG_DATA_DIRS}) { $INSTALL_DIRS = '$(XDG_DATA_DIRS)' }
    else { $INSTALL_DIRS = '$(DESTDIR)$(PREFIX)/share' }

    ###########################################
    ## MO files

    # update mo files
    my $update_mo = "update_mo :\n";
    $update_mo .= "\tmake -C po update-mo\n";
    # install mo files
    my $mopath = '';
    if (exists $ENV{LOCALE_PREFIX}) { 
	$mopath = "$ENV{LOCALE_PREFIX}/share/locale" 
    } else { $mopath = '$(DESTDIR)$(PREFIX)/share/locale' }
    my $mo_install = '';
    my $mo_uninstall = '';
    foreach my $po (<po/*.po>) {
	$po =~ m!po/(\w+)\.po!;
	my $lang = $1;
	my $mo = "po/$lang.mo";
	$mo_install .= "\tinstall -d $mopath/$lang/LC_MESSAGES\n";
 	$mo_install .= "\tinstall -m 644 $mo $mopath/$lang/LC_MESSAGES/gwine.mo\n";
 	$mo_uninstall .= "\trm -f $mopath/$lang/LC_MESSAGES/gwine.mo\n";
    }

    ###########################################
    ## OMF

    # Get subdirs of help directory (ie languages)
    my (@omf, %dir);
    tie %dir, 'IO::Dir', 'help';
    foreach my $file (keys %dir) {
	next if $file eq '.';
	next if $file eq '..';
	next if $file eq 'CVS';
	push @omf, $file;
    }
    untie %dir;
    my $omf = "update_omf :";
    $omf .= " update_omf-$_" foreach @omf;
    $omf .= "\n\n";
    foreach my $l (@omf) {
	my $docdir = "help/$l";
	$omf .= "update_omf-$l:\n";
	$omf .= "\tscrollkeeper-preinstall $INSTALL_DIRS/gnome/help/gwine/$l/gwine.xml $docdir/gwine-$l.omf gwine-$l.omf.out\n";
    }

    # Install / uninstall docs
    # install xml files
    my @doc_install = install_dir('help', "\$(DESTDIR)\$(PREFIX)/share/gnome/help/gwine");
    my $doc_install = '';
    my $doc_uninstall = '';
    foreach (@doc_install) {
	next if /omf/;
	$doc_install .= $_;
    }
    # install updated omf
    my $omfdir = `scrollkeeper-config --omfdir`;
    chomp $omfdir;
    foreach my $l (@omf) {
	$doc_install .= "\tinstall -d \$(DESTDIR)$omfdir/gwine\n";
	$doc_install .= "\tinstall -m 644 gwine-$l.omf.out \$(DESTDIR)$omfdir/gwine/gwine-$l.omf\n";
	$doc_uninstall .= "\trm -f \$(DESTDIR)$omfdir/gwine/gwine-$l.omf\n";
    }
    # update Scrollkeeper database
    my $pkglocalstatedir = `scrollkeeper-config --pkglocalstatedir`;
    chomp $pkglocalstatedir;
    $doc_install .= "\tscrollkeeper-update -p \$(DESTDIR)$pkglocalstatedir -o \$(DESTDIR)$omfdir/gwine\n" unless $::deb_pkg;
    $doc_uninstall .= "\tscrollkeeper-update -p \$(DESTDIR)$pkglocalstatedir\n" unless $::deb_pkg;
    # uninstall 
    foreach (@doc_install) {
	next if m/^\s*install -d/;
	next if m/omf/;
	m/install -m \d+ ([\w\d\/_\.-]+) (.*)$/;
	my $file = $1;
	my $path = $2;
	$doc_uninstall .= "\trm -f $path/$file\n" if (defined $file) && (defined $path);
    }

    ###########################################
    ## Install glade file
    my $path = '$(DESTDIR)$(PREFIX)/share/gwine';
    my $glade_install = "\tinstall -d $path\n";
    $glade_install .= "\tinstall -m 644 gwine.glade $path/gwine.glade\n";
    my $glade_uninstall = "\trm -f $path/gwine.glade\n";

    ###########################################
    ## Install/unistall icons
    $path = '$(DESTDIR)$(PREFIX)/share/gwine/pixmaps';
    my $pixmaps_install = "\tinstall -d $path\n";
    my $pixmaps_uninstall = "";
    foreach my $file (<pixmaps/*.png>) {
	my $newfile = $file;
	$newfile =~ s!pixmaps/!!;
	$pixmaps_install .= "\tinstall -m 644 $file $path/$newfile\n";
	$pixmaps_uninstall .= "\trm -f $path/$newfile\n";	
    }

    ## Install gwine.png in share/pixmaps too (for gwine.desktop)
    $pixmaps_install .= "\tinstall -d \$(DESTDIR)\$(PREFIX)/share/pixmaps\n";
    $pixmaps_install .= "\tinstall -m 644 pixmaps/gwine.png \$(DESTDIR)\$(PREFIX)/share/pixmaps/gwine.png\n";
    $pixmaps_uninstall .= "\trm -f \$(DESTDIR)\$(PREFIX)/share/pixmaps/gwine.png\n";

    ###########################################
    ## Install Free Desktop files
    ## Install gwine.desktop
    my $desktop_install = "\tinstall -d $INSTALL_DIRS/applications/\n";
    $desktop_install .= "\tinstall -m 644 gwine.desktop $INSTALL_DIRS/applications/gwine.desktop\n";
    $desktop_install .= "\tupdate-desktop-database\n" unless $::deb_pkg;
    my $desktop_uninstall = "\trm -f $INSTALL_DIRS/applications/gwine.desktop\n";

    ## Install gwine.xml
    my $mime_install = "\tinstall -d $INSTALL_DIRS/mime/packages/\n";
    $mime_install .= "\tinstall -m 644 gwine.xml $INSTALL_DIRS/mime/packages/gwine.xml\n";
    $mime_install .= "\tupdate-mime-database $INSTALL_DIRS/mime\n" unless $::deb_pkg;
    my $mime_uninstall = "\trm -f $INSTALL_DIRS/mime/packages/gwine.xml\n";

    ###########################################
    ## Install gconf schemas
    my $gconf_install = '';
    my $gconf_uninstall = '';
    if ($::install_gconf_schema) {
	$gconf_install = "\tgconftool --install-schema-file=gwine.schemas\n"
    } elsif ($::deb_pkg) {
	# Place schema in /etc/gconf/schemas (it seems dh_gconf searches here)
	$gconf_install = "\tinstall -d \$(DESTDIR)/etc/gconf/schemas\n";
	$gconf_install .= "\tinstall -m 644 gwine.schemas \$(DESTDIR)/etc/gconf/schemas\n";
	$gconf_uninstall = "\trm \$(DESTDIR)/etc/gconf/schemas\n"
    }

<<EOT;
$update_mo

$omf

post_install: po/fr.mo
$mo_install
$glade_install
$pixmaps_install
$gconf_install
$desktop_install
$mime_install
$doc_install

post_uninstall:
$mo_uninstall
$glade_uninstall
$pixmaps_uninstall
$gconf_uninstall
$desktop_uninstall
$mime_uninstall
$doc_uninstall
EOT
}


exit(0);

# End of Makefile.PL
